@@ -5,8 +5,8 @@
 /*                           GODOT ENGINE                                */
 /*                      https://godotengine.org                          */
 /*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
+/* Copyright (c) 2007-2022 Juan Linietsky, Ariel Manzur.                 */
+/* Copyright (c) 2014-2022 Godot Engine contributors (cf. AUTHORS.md).   */
 /*                                                                       */
 /* Permission is hereby granted, free of charge, to any person obtaining */
 /* a copy of this software and associated documentation files (the       */
@@ -40,11 +40,10 @@
 
 #include <stdarg.h>
 
-OS *OS::singleton = NULL;
+OS *OS::singleton = nullptr;
 uint64_t OS::target_ticks = 0;
 
 OS *OS::get_singleton() {
-
 	return singleton;
 }
 
@@ -67,25 +66,18 @@ String OS::get_iso_date_time(bool local) const {
 		timezone = "Z";
 	}
 
-	return itos(date.year).pad_zeros(2) +
-		   "-" +
-		   itos(date.month).pad_zeros(2) +
-		   "-" +
-		   itos(date.day).pad_zeros(2) +
-		   "T" +
-		   itos(time.hour).pad_zeros(2) +
-		   ":" +
-		   itos(time.min).pad_zeros(2) +
-		   ":" +
-		   itos(time.sec).pad_zeros(2) +
-		   timezone;
+	return itos(date.year).pad_zeros(2) + "-" +
+			itos(date.month).pad_zeros(2) + "-" +
+			itos(date.day).pad_zeros(2) + "T" +
+			itos(time.hour).pad_zeros(2) + ":" +
+			itos(time.min).pad_zeros(2) + ":" +
+			itos(time.sec).pad_zeros(2) + timezone;
 }
 
 uint64_t OS::get_splash_tick_msec() const {
 	return _msec_splash;
 }
 uint64_t OS::get_unix_time() const {
-
 	return 0;
 };
 uint64_t OS::get_system_time_secs() const {
@@ -94,6 +86,9 @@ uint64_t OS::get_system_time_secs() const {
 uint64_t OS::get_system_time_msecs() const {
 	return 0;
 }
+double OS::get_subsecond_unix_time() const {
+	return 0.0;
+}
 void OS::debug_break(){
 
 	// something
@@ -117,12 +112,10 @@ void OS::add_logger(Logger *p_logger) {
 }
 
 void OS::print_error(const char *p_function, const char *p_file, int p_line, const char *p_code, const char *p_rationale, Logger::ErrorType p_type) {
-
 	_logger->log_error(p_function, p_file, p_line, p_code, p_rationale, p_type);
 }
 
 void OS::print(const char *p_format, ...) {
-
 	va_list argp;
 	va_start(argp, p_format);
 
@@ -149,51 +142,58 @@ bool OS::is_keep_screen_on() const {
 }
 
 void OS::set_low_processor_usage_mode(bool p_enabled) {
-
 	low_processor_usage_mode = p_enabled;
 }
 
 bool OS::is_in_low_processor_usage_mode() const {
-
 	return low_processor_usage_mode;
 }
 
-void OS::set_low_processor_usage_mode_sleep_usec(int p_usec) {
+void OS::set_update_vital_only(bool p_enabled) {
+	_update_vital_only = p_enabled;
+}
 
+void OS::set_low_processor_usage_mode_sleep_usec(int p_usec) {
 	low_processor_usage_mode_sleep_usec = p_usec;
 }
 
 int OS::get_low_processor_usage_mode_sleep_usec() const {
-
 	return low_processor_usage_mode_sleep_usec;
 }
 
 void OS::set_clipboard(const String &p_text) {
-
 	_local_clipboard = p_text;
 }
-String OS::get_clipboard() const {
 
+String OS::get_clipboard() const {
 	return _local_clipboard;
 }
 
-String OS::get_executable_path() const {
+bool OS::has_clipboard() const {
+	return !get_clipboard().empty();
+}
 
+void OS::set_clipboard_primary(const String &p_text) {
+	_primary_clipboard = p_text;
+}
+
+String OS::get_clipboard_primary() const {
+	return _primary_clipboard;
+}
+
+String OS::get_executable_path() const {
 	return _execpath;
 }
 
 int OS::get_process_id() const {
-
 	return -1;
 };
 
 void OS::vibrate_handheld(int p_duration_ms) {
-
-	WARN_PRINTS("vibrate_handheld() only works with Android and iOS");
+	WARN_PRINT("vibrate_handheld() only works with Android, iOS and HTML5");
 }
 
 bool OS::is_stdout_verbose() const {
-
 	return _verbose_stdout;
 }
 
@@ -202,27 +202,26 @@ bool OS::is_stdout_debug_enabled() const {
 }
 
 void OS::dump_memory_to_file(const char *p_file) {
-
 	//Memory::dump_static_mem_to_file(p_file);
 }
 
-static FileAccess *_OSPRF = NULL;
+static FileAccess *_OSPRF = nullptr;
 
 static void _OS_printres(Object *p_obj) {
-
 	Resource *res = Object::cast_to<Resource>(p_obj);
-	if (!res)
+	if (!res) {
 		return;
+	}
 
-	String str = itos(res->get_instance_id()) + String(res->get_class()) + ":" + String(res->get_name()) + " - " + res->get_path();
-	if (_OSPRF)
+	String str = vformat("%s - %s - %s", res->to_string(), res->get_name(), res->get_path());
+	if (_OSPRF) {
 		_OSPRF->store_line(str);
-	else
+	} else {
 		print_line(str);
+	}
 }
 
 bool OS::has_virtual_keyboard() const {
-
 	return false;
 }
 
@@ -236,6 +235,10 @@ int OS::get_virtual_keyboard_height() const {
 	return 0;
 }
 
+uint32_t OS::keyboard_get_scancode_from_physical(uint32_t p_scancode) const {
+	return p_scancode;
+}
+
 void OS::set_cursor_shape(CursorShape p_shape) {
 }
 
@@ -247,14 +250,12 @@ void OS::set_custom_mouse_cursor(const RES &p_cursor, CursorShape p_shape, const
 }
 
 void OS::print_all_resources(String p_to_file) {
-
 	ERR_FAIL_COND(p_to_file != "" && _OSPRF);
 	if (p_to_file != "") {
-
 		Error err;
 		_OSPRF = FileAccess::open(p_to_file, FileAccess::WRITE, &err);
 		if (err != OK) {
-			_OSPRF = NULL;
+			_OSPRF = nullptr;
 			ERR_FAIL_MSG("Can't print all resources to file: " + String(p_to_file) + ".");
 		}
 	}
@@ -262,50 +263,58 @@ void OS::print_all_resources(String p_to_file) {
 	ObjectDB::debug_objects(_OS_printres);
 
 	if (p_to_file != "") {
-
-		if (_OSPRF)
+		if (_OSPRF) {
 			memdelete(_OSPRF);
-		_OSPRF = NULL;
+		}
+		_OSPRF = nullptr;
 	}
 }
 
 void OS::print_resources_in_use(bool p_short) {
-
-	ResourceCache::dump(NULL, p_short);
+	ResourceCache::dump(nullptr, p_short);
 }
 
 void OS::dump_resources_to_file(const char *p_file) {
-
 	ResourceCache::dump(p_file);
 }
 
 void OS::set_no_window_mode(bool p_enable) {
-
 	_no_window = p_enable;
 }
 
 bool OS::is_no_window_mode_enabled() const {
-
 	return _no_window;
 }
 
 int OS::get_exit_code() const {
-
 	return _exit_code;
 }
 void OS::set_exit_code(int p_code) {
-
 	_exit_code = p_code;
+	_is_custom_exit_code = true;
+}
+
+bool OS::is_custom_exit_code() {
+	return _is_custom_exit_code;
 }
 
 String OS::get_locale() const {
-
 	return "en";
 }
 
+// Non-virtual helper to extract the 2 or 3-letter language code from
+// `get_locale()` in a way that's consistent for all platforms.
+String OS::get_locale_language() const {
+	return get_locale().left(3).replace("_", "");
+}
+
+// Embedded PCK offset.
+uint64_t OS::get_embedded_pck_offset() const {
+	return 0;
+}
+
 // Helper function to ensure that a dir name/path will be valid on the OS
 String OS::get_safe_dir_name(const String &p_dir_name, bool p_allow_dir_separator) const {
-
 	Vector<String> invalid_chars = String(": * ? \" < > |").split(" ");
 	if (p_allow_dir_separator) {
 		// Dir separators are allowed, but disallow ".." to avoid going up the filesystem
@@ -325,50 +334,47 @@ String OS::get_safe_dir_name(const String &p_dir_name, bool p_allow_dir_separato
 
 // Get properly capitalized engine name for system paths
 String OS::get_godot_dir_name() const {
-
 	// Default to lowercase, so only override when different case is needed
 	return String(VERSION_SHORT_NAME).to_lower();
 }
 
 // OS equivalent of XDG_DATA_HOME
 String OS::get_data_path() const {
-
 	return ".";
 }
 
 // OS equivalent of XDG_CONFIG_HOME
 String OS::get_config_path() const {
-
 	return ".";
 }
 
 // OS equivalent of XDG_CACHE_HOME
 String OS::get_cache_path() const {
-
 	return ".";
 }
 
 // Path to macOS .app bundle resources
 String OS::get_bundle_resource_dir() const {
-
 	return ".";
 };
 
+// Path to macOS .app bundle embedded icon
+String OS::get_bundle_icon_path() const {
+	return String();
+}
+
 // OS specific path for user://
 String OS::get_user_data_dir() const {
-
 	return ".";
 };
 
 // Absolute path to res://
 String OS::get_resource_dir() const {
-
 	return ProjectSettings::get_singleton()->get_resource_path();
 }
 
 // Access system-specific dirs like Documents, Downloads, etc.
-String OS::get_system_dir(SystemDir p_dir) const {
-
+String OS::get_system_dir(SystemDir p_dir, bool p_shared_storage) const {
 	return ".";
 }
 
@@ -378,30 +384,32 @@ Error OS::shell_open(String p_uri) {
 
 // implement these with the canvas?
 Error OS::dialog_show(String p_title, String p_description, Vector<String> p_buttons, Object *p_obj, String p_callback) {
-
 	while (true) {
-
 		print("%ls\n--------\n%ls\n", p_title.c_str(), p_description.c_str());
 		for (int i = 0; i < p_buttons.size(); i++) {
-			if (i > 0) print(", ");
+			if (i > 0) {
+				print(", ");
+			}
 			print("%i=%ls", i + 1, p_buttons[i].c_str());
 		};
 		print("\n");
 		String res = get_stdin_string().strip_edges();
-		if (!res.is_numeric())
+		if (!res.is_numeric()) {
 			continue;
+		}
 		int n = res.to_int();
-		if (n < 0 || n >= p_buttons.size())
+		if (n < 0 || n >= p_buttons.size()) {
 			continue;
-		if (p_obj && p_callback != "")
+		}
+		if (p_obj && p_callback != "") {
 			p_obj->call_deferred(p_callback, n);
+		}
 		break;
 	};
 	return OK;
 };
 
 Error OS::dialog_input_text(String p_title, String p_description, String p_partial, Object *p_obj, String p_callback) {
-
 	ERR_FAIL_COND_V(!p_obj, FAILED);
 	ERR_FAIL_COND_V(p_callback == "", FAILED);
 	print("%ls\n---------\n%ls\n[%ls]:\n", p_title.c_str(), p_description.c_str(), p_partial.c_str());
@@ -418,32 +426,26 @@ Error OS::dialog_input_text(String p_title, String p_description, String p_parti
 };
 
 uint64_t OS::get_static_memory_usage() const {
-
 	return Memory::get_mem_usage();
 }
 uint64_t OS::get_dynamic_memory_usage() const {
-
 	return MemoryPool::total_memory;
 }
 
 uint64_t OS::get_static_memory_peak_usage() const {
-
 	return Memory::get_mem_max_usage();
 }
 
 Error OS::set_cwd(const String &p_cwd) {
-
 	return ERR_CANT_OPEN;
 }
 
 bool OS::has_touchscreen_ui_hint() const {
-
 	//return false;
 	return Input::get_singleton() && Input::get_singleton()->is_emulating_touch_from_mouse();
 }
 
 uint64_t OS::get_free_static_memory() const {
-
 	return Memory::get_mem_available();
 }
 
@@ -451,17 +453,35 @@ void OS::yield() {
 }
 
 void OS::set_screen_orientation(ScreenOrientation p_orientation) {
-
 	_orientation = p_orientation;
 }
 
 OS::ScreenOrientation OS::get_screen_orientation() const {
-
 	return (OS::ScreenOrientation)_orientation;
 }
 
-void OS::_ensure_user_data_dir() {
+// Internal helper function that returns the screen orientation enum value from a string
+// (generally coming from the Project Settings).
+// This is required to keep compatibility with existing projects.
+OS::ScreenOrientation OS::get_screen_orientation_from_string(const String &p_orientation) const {
+	if (p_orientation == "portrait") {
+		return OS::SCREEN_PORTRAIT;
+	} else if (p_orientation == "reverse_landscape") {
+		return OS::SCREEN_REVERSE_LANDSCAPE;
+	} else if (p_orientation == "reverse_portrait") {
+		return OS::SCREEN_REVERSE_PORTRAIT;
+	} else if (p_orientation == "sensor_landscape") {
+		return OS::SCREEN_SENSOR_LANDSCAPE;
+	} else if (p_orientation == "sensor_portrait") {
+		return OS::SCREEN_SENSOR_PORTRAIT;
+	} else if (p_orientation == "sensor") {
+		return OS::SCREEN_SENSOR;
+	}
 
+	return OS::SCREEN_LANDSCAPE;
+}
+
+void OS::ensure_user_data_dir() {
 	String dd = get_user_data_dir();
 	DirAccess *da = DirAccess::open(dd);
 	if (da) {
@@ -483,13 +503,11 @@ void OS::set_icon(const Ref<Image> &p_icon) {
 }
 
 String OS::get_model_name() const {
-
 	return "GenericDevice";
 }
 
 void OS::set_cmdline(const char *p_execpath, const List<String> &p_args) {
-
-	_execpath = p_execpath;
+	_execpath = String::utf8(p_execpath);
 	_cmdline = p_args;
 };
 
@@ -503,42 +521,38 @@ void OS::swap_buffers() {
 }
 
 String OS::get_unique_id() const {
-
 	ERR_FAIL_V("");
 }
 
 int OS::get_processor_count() const {
-
 	return 1;
 }
 
-Error OS::native_video_play(String p_path, float p_volume, String p_audio_track, String p_subtitle_track) {
+String OS::get_processor_name() const {
+	return "";
+}
 
+Error OS::native_video_play(String p_path, float p_volume, String p_audio_track, String p_subtitle_track) {
 	return FAILED;
-};
+}
 
 bool OS::native_video_is_playing() const {
-
 	return false;
-};
+}
 
-void OS::native_video_pause(){
+void OS::native_video_pause() {
+}
 
-};
+void OS::native_video_unpause() {
+}
 
-void OS::native_video_unpause(){
-
-};
-
-void OS::native_video_stop(){
-
-};
+void OS::native_video_stop() {
+}
 
 void OS::set_mouse_mode(MouseMode p_mode) {
 }
 
 bool OS::can_use_threads() const {
-
 #ifdef NO_THREADS
 	return false;
 #else
@@ -547,12 +561,10 @@ bool OS::can_use_threads() const {
 }
 
 OS::MouseMode OS::get_mouse_mode() const {
-
 	return MOUSE_MODE_VISIBLE;
 }
 
 OS::LatinKeyboardVariant OS::get_latin_keyboard_variant() const {
-
 	return LATIN_KEYBOARD_QWERTY;
 }
 
@@ -585,7 +597,7 @@ String OS::get_joy_guid(int p_device) const {
 void OS::set_context(int p_context) {
 }
 
-OS::SwitchVSyncCallbackInThread OS::switch_vsync_function = NULL;
+OS::SwitchVSyncCallbackInThread OS::switch_vsync_function = nullptr;
 
 void OS::set_use_vsync(bool p_enable) {
 	_use_vsync = p_enable;
@@ -597,7 +609,6 @@ void OS::set_use_vsync(bool p_enable) {
 }
 
 bool OS::is_vsync_enabled() const {
-
 	return _use_vsync;
 }
 
@@ -609,6 +620,14 @@ bool OS::is_vsync_via_compositor_enabled() const {
 	return _vsync_via_compositor;
 }
 
+void OS::set_delta_smoothing(bool p_enabled) {
+	_delta_smoothing_enabled = p_enabled;
+}
+
+bool OS::is_delta_smoothing_enabled() const {
+	return _delta_smoothing_enabled;
+}
+
 OS::PowerState OS::get_power_state() {
 	return POWERSTATE_UNKNOWN;
 }
@@ -620,27 +639,30 @@ int OS::get_power_percent_left() {
 }
 
 void OS::set_has_server_feature_callback(HasServerFeatureCallback p_callback) {
-
 	has_server_feature_callback = p_callback;
 }
 
 bool OS::has_feature(const String &p_feature) {
-
-	if (p_feature == get_name())
+	if (p_feature == get_name()) {
 		return true;
+	}
 #ifdef DEBUG_ENABLED
-	if (p_feature == "debug")
+	if (p_feature == "debug") {
 		return true;
+	}
 #else
-	if (p_feature == "release")
+	if (p_feature == "release") {
 		return true;
+	}
 #endif
 #ifdef TOOLS_ENABLED
-	if (p_feature == "editor")
+	if (p_feature == "editor") {
 		return true;
+	}
 #else
-	if (p_feature == "standalone")
+	if (p_feature == "standalone") {
 		return true;
+	}
 #endif
 
 	if (sizeof(void *) == 8 && p_feature == "64") {
@@ -649,19 +671,25 @@ bool OS::has_feature(const String &p_feature) {
 	if (sizeof(void *) == 4 && p_feature == "32") {
 		return true;
 	}
-#if defined(__x86_64) || defined(__x86_64__) || defined(__amd64__)
+#if defined(__x86_64) || defined(__x86_64__) || defined(__amd64__) || defined(_M_X64)
 	if (p_feature == "x86_64") {
 		return true;
 	}
-#elif (defined(__i386) || defined(__i386__))
+#elif defined(__i386) || defined(__i386__) || defined(_M_IX86)
+	if (p_feature == "x86_32") {
+		return true;
+	}
 	if (p_feature == "x86") {
 		return true;
 	}
-#elif defined(__aarch64__)
+#elif defined(__aarch64__) || defined(_M_ARM64)
 	if (p_feature == "arm64") {
 		return true;
 	}
-#elif defined(__arm__)
+#elif defined(__arm__) || defined(_M_ARM)
+	if (p_feature == "arm32") {
+		return true;
+	}
 #if defined(__ARM_ARCH_7A__)
 	if (p_feature == "armv7a" || p_feature == "armv7") {
 		return true;
@@ -675,24 +703,58 @@ bool OS::has_feature(const String &p_feature) {
 	if (p_feature == "arm") {
 		return true;
 	}
+#elif defined(__riscv)
+#if __riscv_xlen == 8
+	if (p_feature == "rv64") {
+		return true;
+	}
+#endif
+	if (p_feature == "riscv") {
+		return true;
+	}
+#elif defined(__powerpc__)
+#if defined(__powerpc64__)
+	if (p_feature == "ppc64") {
+		return true;
+	}
+#endif
+	if (p_feature == "ppc") {
+		return true;
+	}
+#elif defined(__wasm__)
+#if defined(__wasm64__)
+	if (p_feature == "wasm64") {
+		return true;
+	}
+#elif defined(__wasm32__)
+	if (p_feature == "wasm32") {
+		return true;
+	}
+#endif
+	if (p_feature == "wasm") {
+		return true;
+	}
 #endif
 
-	if (_check_internal_feature_support(p_feature))
+	if (_check_internal_feature_support(p_feature)) {
 		return true;
+	}
 
 	if (has_server_feature_callback && has_server_feature_callback(p_feature)) {
 		return true;
 	}
 
-	if (ProjectSettings::get_singleton()->has_custom_feature(p_feature))
+	if (ProjectSettings::get_singleton()->has_custom_feature(p_feature)) {
 		return true;
+	}
 
 	return false;
 }
 
 void OS::center_window() {
-
-	if (is_window_fullscreen()) return;
+	if (is_window_fullscreen()) {
+		return;
+	}
 
 	Point2 sp = get_screen_position(get_current_screen());
 	Size2 scr = get_screen_size(get_current_screen());
@@ -705,12 +767,10 @@ void OS::center_window() {
 }
 
 int OS::get_video_driver_count() const {
-
 	return 2;
 }
 
 const char *OS::get_video_driver_name(int p_driver) const {
-
 	switch (p_driver) {
 		case VIDEO_DRIVER_GLES2:
 			return "GLES2";
@@ -720,13 +780,17 @@ const char *OS::get_video_driver_name(int p_driver) const {
 	}
 }
 
-int OS::get_audio_driver_count() const {
+bool OS::is_offscreen_gl_available() const {
+	return false;
+}
 
+void OS::set_offscreen_gl_current(bool p_current) {}
+
+int OS::get_audio_driver_count() const {
 	return AudioDriverManager::get_driver_count();
 }
 
 const char *OS::get_audio_driver_name(int p_driver) const {
-
 	AudioDriver *driver = AudioDriverManager::get_driver(p_driver);
 	ERR_FAIL_COND_V_MSG(!driver, "", "Cannot get audio driver at index '" + itos(p_driver) + "'.");
 	return AudioDriverManager::get_driver(p_driver)->get_name();
@@ -746,16 +810,15 @@ List<String> OS::get_restart_on_exit_arguments() const {
 }
 
 PoolStringArray OS::get_connected_midi_inputs() {
-
-	if (MIDIDriver::get_singleton())
+	if (MIDIDriver::get_singleton()) {
 		return MIDIDriver::get_singleton()->get_connected_inputs();
+	}
 
 	PoolStringArray list;
 	ERR_FAIL_V_MSG(list, vformat("MIDI input isn't supported on %s.", OS::get_singleton()->get_name()));
 }
 
 void OS::open_midi_inputs() {
-
 	if (MIDIDriver::get_singleton()) {
 		MIDIDriver::get_singleton()->open();
 	} else {
@@ -764,7 +827,6 @@ void OS::open_midi_inputs() {
 }
 
 void OS::close_midi_inputs() {
-
 	if (MIDIDriver::get_singleton()) {
 		MIDIDriver::get_singleton()->close();
 	} else {
@@ -815,11 +877,14 @@ OS::OS() {
 	_keep_screen_on = true; // set default value to true, because this had been true before godot 2.0.
 	low_processor_usage_mode = false;
 	low_processor_usage_mode_sleep_usec = 10000;
+	_update_vital_only = false;
+	_update_pending = false;
 	_verbose_stdout = false;
 	_debug_stdout = false;
 	_no_window = false;
 	_exit_code = 0;
 	_orientation = SCREEN_LANDSCAPE;
+	_delta_smoothing_enabled = false;
 
 	_render_thread_mode = RENDER_THREAD_SAFE;
 
@@ -827,9 +892,9 @@ OS::OS() {
 	_allow_layered = false;
 	_stack_bottom = (void *)(&stack_bottom);
 
-	_logger = NULL;
+	_logger = nullptr;
 
-	has_server_feature_callback = NULL;
+	has_server_feature_callback = nullptr;
 
 	Vector<Logger *> loggers;
 	loggers.push_back(memnew(StdLogger));
@@ -838,5 +903,5 @@ OS::OS() {
 
 OS::~OS() {
 	memdelete(_logger);
-	singleton = NULL;
+	singleton = nullptr;
 }
