@@ -5,8 +5,8 @@
 /*                           GODOT ENGINE                                */
 /*                      https://godotengine.org                          */
 /*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
+/* Copyright (c) 2007-2022 Juan Linietsky, Ariel Manzur.                 */
+/* Copyright (c) 2014-2022 Godot Engine contributors (cf. AUTHORS.md).   */
 /*                                                                       */
 /* Permission is hereby granted, free of charge, to any person obtaining */
 /* a copy of this software and associated documentation files (the       */
@@ -34,6 +34,13 @@
 #include "editor/editor_settings.h"
 #include "editor/plugins/animation_player_editor_plugin.h"
 
+void InspectorDock::_prepare_menu() {
+	PopupMenu *menu = object_menu->get_popup();
+	for (int i = EditorPropertyNameProcessor::STYLE_RAW; i <= EditorPropertyNameProcessor::STYLE_LOCALIZED; i++) {
+		menu->set_item_checked(menu->get_item_index(PROPERTY_NAME_STYLE_RAW + i), i == property_name_style);
+	}
+}
+
 void InspectorDock::_menu_option(int p_option) {
 	switch (p_option) {
 		case EXPAND_ALL: {
@@ -42,6 +49,14 @@ void InspectorDock::_menu_option(int p_option) {
 		case COLLAPSE_ALL: {
 			_menu_collapseall();
 		} break;
+
+		case RESOURCE_SAVE: {
+			_save_resource(false);
+		} break;
+		case RESOURCE_SAVE_AS: {
+			_save_resource(true);
+		} break;
+
 		case RESOURCE_MAKE_BUILT_IN: {
 			_unref_resource();
 		} break;
@@ -52,13 +67,6 @@ void InspectorDock::_menu_option(int p_option) {
 			_paste_resource();
 		} break;
 
-		case RESOURCE_SAVE: {
-			_save_resource(false);
-		} break;
-		case RESOURCE_SAVE_AS: {
-			_save_resource(true);
-		} break;
-
 		case OBJECT_REQUEST_HELP: {
 			if (current) {
 				editor->set_visible_editor(EditorNode::EDITOR_SCRIPT);
@@ -68,14 +76,16 @@ void InspectorDock::_menu_option(int p_option) {
 
 		case OBJECT_COPY_PARAMS: {
 			editor_data->apply_changes_in_editors();
-			if (current)
+			if (current) {
 				editor_data->copy_object_params(current);
+			}
 		} break;
 
 		case OBJECT_PASTE_PARAMS: {
 			editor_data->apply_changes_in_editors();
-			if (current)
+			if (current) {
 				editor_data->paste_object_params(current);
+			}
 		} break;
 
 		case OBJECT_UNIQUE_RESOURCES: {
@@ -85,18 +95,16 @@ void InspectorDock::_menu_option(int p_option) {
 				current->get_property_list(&props);
 				Map<RES, RES> duplicates;
 				for (List<PropertyInfo>::Element *E = props.front(); E; E = E->next()) {
-
-					if (!(E->get().usage & PROPERTY_USAGE_STORAGE))
+					if (!(E->get().usage & PROPERTY_USAGE_STORAGE)) {
 						continue;
+					}
 
 					Variant v = current->get(E->get().name);
 					if (v.is_ref()) {
 						REF ref = v;
 						if (ref.is_valid()) {
-
 							RES res = ref;
 							if (res.is_valid()) {
-
 								if (!duplicates.has(res)) {
 									duplicates[res] = res->duplicate();
 								}
@@ -112,11 +120,18 @@ void InspectorDock::_menu_option(int p_option) {
 
 			editor_data->get_undo_redo().clear_history();
 
-			editor->get_editor_plugins_over()->edit(NULL);
+			editor->get_editor_plugins_over()->edit(nullptr);
 			editor->get_editor_plugins_over()->edit(current);
 
 		} break;
 
+		case PROPERTY_NAME_STYLE_RAW:
+		case PROPERTY_NAME_STYLE_CAPITALIZED:
+		case PROPERTY_NAME_STYLE_LOCALIZED: {
+			property_name_style = (EditorPropertyNameProcessor::Style)(p_option - PROPERTY_NAME_STYLE_RAW);
+			inspector->set_property_name_style(property_name_style);
+		} break;
+
 		default: {
 			if (p_option >= OBJECT_METHOD_BASE) {
 				ERR_FAIL_COND(!current);
@@ -166,21 +181,22 @@ void InspectorDock::_resource_file_selected(String p_file) {
 
 void InspectorDock::_save_resource(bool save_as) const {
 	uint32_t current = EditorNode::get_singleton()->get_editor_history()->get_current();
-	Object *current_obj = current > 0 ? ObjectDB::get_instance(current) : NULL;
+	Object *current_obj = current > 0 ? ObjectDB::get_instance(current) : nullptr;
 
 	ERR_FAIL_COND(!Object::cast_to<Resource>(current_obj));
 
 	RES current_res = RES(Object::cast_to<Resource>(current_obj));
 
-	if (save_as)
+	if (save_as) {
 		editor->save_resource_as(current_res);
-	else
+	} else {
 		editor->save_resource(current_res);
+	}
 }
 
 void InspectorDock::_unref_resource() const {
 	uint32_t current = EditorNode::get_singleton()->get_editor_history()->get_current();
-	Object *current_obj = current > 0 ? ObjectDB::get_instance(current) : NULL;
+	Object *current_obj = current > 0 ? ObjectDB::get_instance(current) : nullptr;
 
 	ERR_FAIL_COND(!Object::cast_to<Resource>(current_obj));
 
@@ -191,7 +207,7 @@ void InspectorDock::_unref_resource() const {
 
 void InspectorDock::_copy_resource() const {
 	uint32_t current = EditorNode::get_singleton()->get_editor_history()->get_current();
-	Object *current_obj = current > 0 ? ObjectDB::get_instance(current) : NULL;
+	Object *current_obj = current > 0 ? ObjectDB::get_instance(current) : nullptr;
 
 	ERR_FAIL_COND(!Object::cast_to<Resource>(current_obj));
 
@@ -207,6 +223,12 @@ void InspectorDock::_paste_resource() const {
 	}
 }
 
+void InspectorDock::_prepare_resource_extra_popup() {
+	RES r = EditorSettings::get_singleton()->get_resource_clipboard();
+	PopupMenu *popup = resource_extra_button->get_popup();
+	popup->set_item_disabled(popup->get_item_index(RESOURCE_EDIT_CLIPBOARD), r.is_null());
+}
+
 void InspectorDock::_prepare_history() {
 	EditorHistory *editor_history = EditorNode::get_singleton()->get_editor_history();
 
@@ -217,7 +239,6 @@ void InspectorDock::_prepare_history() {
 	Ref<Texture> base_icon = get_icon("Object", "EditorIcons");
 	Set<ObjectID> already;
 	for (int i = editor_history->get_history_len() - 1; i >= history_to; i--) {
-
 		ObjectID id = editor_history->get_history_obj(i);
 		Object *obj = ObjectDB::get_instance(id);
 		if (!obj || already.has(id)) {
@@ -237,9 +258,9 @@ void InspectorDock::_prepare_history() {
 		String text;
 		if (Object::cast_to<Resource>(obj)) {
 			Resource *r = Object::cast_to<Resource>(obj);
-			if (r->get_path().is_resource_file())
+			if (r->get_path().is_resource_file()) {
 				text = r->get_path().get_file();
-			else if (r->get_name() != String()) {
+			} else if (r->get_name() != String()) {
 				text = r->get_name();
 			} else {
 				text = r->get_class();
@@ -263,38 +284,41 @@ void InspectorDock::_select_history(int p_idx) const {
 	//push it to the top, it is not correct, but it's more useful
 	ObjectID id = EditorNode::get_singleton()->get_editor_history()->get_history_obj(p_idx);
 	Object *obj = ObjectDB::get_instance(id);
-	if (!obj)
+	if (!obj) {
 		return;
+	}
 	editor->push_item(obj);
 }
 
 void InspectorDock::_resource_created() const {
-	Object *c = new_resource_dialog->instance_selected();
+	Variant c = new_resource_dialog->instance_selected();
 
 	ERR_FAIL_COND(!c);
 	Resource *r = Object::cast_to<Resource>(c);
 	ERR_FAIL_COND(!r);
 
-	REF res(r);
-	editor->push_item(c);
+	editor->push_item(r);
 }
 
 void InspectorDock::_resource_selected(const RES &p_res, const String &p_property) const {
-	if (p_res.is_null())
+	if (p_res.is_null()) {
 		return;
+	}
 
 	RES r = p_res;
 	editor->push_item(r.operator->(), p_property);
 }
 
 void InspectorDock::_edit_forward() {
-	if (EditorNode::get_singleton()->get_editor_history()->next())
+	if (EditorNode::get_singleton()->get_editor_history()->next()) {
 		editor->edit_current();
+	}
 }
 void InspectorDock::_edit_back() {
 	EditorHistory *editor_history = EditorNode::get_singleton()->get_editor_history();
-	if ((current && editor_history->previous()) || editor_history->get_path_size() == 1)
+	if ((current && editor_history->previous()) || editor_history->get_path_size() == 1) {
 		editor->edit_current();
+	}
 }
 
 void InspectorDock::_menu_collapseall() {
@@ -311,8 +335,9 @@ void InspectorDock::_property_keyed(const String &p_keyed, const Variant &p_valu
 
 void InspectorDock::_transform_keyed(Object *sp, const String &p_sub, const Transform &p_key) {
 	Spatial *s = Object::cast_to<Spatial>(sp);
-	if (!s)
+	if (!s) {
 		return;
+	}
 	AnimationPlayerEditor::singleton->get_track_editor()->insert_transform_key(s, p_sub, p_key);
 }
 
@@ -328,19 +353,29 @@ void InspectorDock::_notification(int p_what) {
 	switch (p_what) {
 		case EditorSettings::NOTIFICATION_EDITOR_SETTINGS_CHANGED: {
 			set_theme(editor->get_gui_base()->get_theme());
+
 			resource_new_button->set_icon(get_icon("New", "EditorIcons"));
 			resource_load_button->set_icon(get_icon("Load", "EditorIcons"));
 			resource_save_button->set_icon(get_icon("Save", "EditorIcons"));
+			resource_extra_button->set_icon(get_icon("GuiTabMenuHl", "EditorIcons"));
+
+			PopupMenu *resource_extra_popup = resource_extra_button->get_popup();
+			resource_extra_popup->set_item_icon(resource_extra_popup->get_item_index(RESOURCE_EDIT_CLIPBOARD), get_icon("ActionPaste", "EditorIcons"));
+			resource_extra_popup->set_item_icon(resource_extra_popup->get_item_index(RESOURCE_COPY), get_icon("ActionCopy", "EditorIcons"));
+
 			backward_button->set_icon(get_icon("Back", "EditorIcons"));
 			forward_button->set_icon(get_icon("Forward", "EditorIcons"));
+
 			history_menu->set_icon(get_icon("History", "EditorIcons"));
 			object_menu->set_icon(get_icon("Tools", "EditorIcons"));
 			warning->set_icon(get_icon("NodeWarning", "EditorIcons"));
+			warning->add_color_override("font_color", get_color("warning_color", "Editor"));
 		} break;
 	}
 }
 
 void InspectorDock::_bind_methods() {
+	ClassDB::bind_method("_prepare_menu", &InspectorDock::_prepare_menu);
 	ClassDB::bind_method("_menu_option", &InspectorDock::_menu_option);
 
 	ClassDB::bind_method("update_keying", &InspectorDock::update_keying);
@@ -353,6 +388,7 @@ void InspectorDock::_bind_methods() {
 	ClassDB::bind_method("_unref_resource", &InspectorDock::_unref_resource);
 	ClassDB::bind_method("_paste_resource", &InspectorDock::_paste_resource);
 	ClassDB::bind_method("_copy_resource", &InspectorDock::_copy_resource);
+	ClassDB::bind_method("_prepare_resource_extra_popup", &InspectorDock::_prepare_resource_extra_popup);
 
 	ClassDB::bind_method("_select_history", &InspectorDock::_select_history);
 	ClassDB::bind_method("_prepare_history", &InspectorDock::_prepare_history);
@@ -387,7 +423,6 @@ void InspectorDock::clear() {
 }
 
 void InspectorDock::update(Object *p_object) {
-
 	EditorHistory *editor_history = EditorNode::get_singleton()->get_editor_history();
 	backward_button->set_disabled(editor_history->is_at_beginning());
 	forward_button->set_disabled(editor_history->is_at_end());
@@ -400,65 +435,61 @@ void InspectorDock::update(Object *p_object) {
 
 	current = p_object;
 
-	if (!p_object) {
-		object_menu->set_disabled(true);
+	const bool is_object = p_object != nullptr;
+	const bool is_resource = is_object && p_object->is_class("Resource");
+	const bool is_node = is_object && p_object->is_class("Node");
+
+	object_menu->set_disabled(!is_object);
+	search->set_editable(is_object);
+	resource_save_button->set_disabled(!is_resource);
+	open_docs_button->set_disabled(!is_resource && !is_node);
+
+	PopupMenu *resource_extra_popup = resource_extra_button->get_popup();
+	resource_extra_popup->set_item_disabled(resource_extra_popup->get_item_index(RESOURCE_COPY), !is_resource);
+	resource_extra_popup->set_item_disabled(resource_extra_popup->get_item_index(RESOURCE_MAKE_BUILT_IN), !is_resource);
+
+	if (!is_object) {
 		warning->hide();
-		search->set_editable(false);
-
-		editor_path->set_disabled(true);
-		editor_path->set_text("");
-		editor_path->set_tooltip("");
-		editor_path->set_icon(NULL);
-
+		editor_path->clear_path();
 		return;
 	}
 
-	bool is_resource = p_object->is_class("Resource");
-	bool is_node = p_object->is_class("Node");
-
-	object_menu->set_disabled(false);
-	search->set_editable(true);
-	editor_path->set_disabled(false);
-	resource_save_button->set_disabled(!is_resource);
+	editor_path->enable_path();
 
 	PopupMenu *p = object_menu->get_popup();
 
 	p->clear();
-	p->add_shortcut(ED_SHORTCUT("property_editor/expand_all", TTR("Expand All Properties")), EXPAND_ALL);
-	p->add_shortcut(ED_SHORTCUT("property_editor/collapse_all", TTR("Collapse All Properties")), COLLAPSE_ALL);
-	p->add_separator();
-	if (is_resource) {
-		p->add_item(TTR("Save"), RESOURCE_SAVE);
-		p->add_item(TTR("Save As..."), RESOURCE_SAVE_AS);
-		p->add_separator();
-	}
-	p->add_shortcut(ED_SHORTCUT("property_editor/copy_params", TTR("Copy Params")), OBJECT_COPY_PARAMS);
-	p->add_shortcut(ED_SHORTCUT("property_editor/paste_params", TTR("Paste Params")), OBJECT_PASTE_PARAMS);
-	p->add_separator();
+	p->add_icon_shortcut(get_icon("GuiTreeArrowDown", "EditorIcons"), ED_SHORTCUT("property_editor/expand_all", TTR("Expand All")), EXPAND_ALL);
+	p->add_icon_shortcut(get_icon("GuiTreeArrowRight", "EditorIcons"), ED_SHORTCUT("property_editor/collapse_all", TTR("Collapse All")), COLLAPSE_ALL);
 
-	p->add_shortcut(ED_SHORTCUT("property_editor/paste_resource", TTR("Edit Resource Clipboard")), RESOURCE_EDIT_CLIPBOARD);
-	if (is_resource) {
-		p->add_shortcut(ED_SHORTCUT("property_editor/copy_resource", TTR("Copy Resource")), RESOURCE_COPY);
-		p->add_shortcut(ED_SHORTCUT("property_editor/unref_resource", TTR("Make Built-In")), RESOURCE_MAKE_BUILT_IN);
+	p->add_separator(TTR("Property Name Style"));
+	p->add_radio_check_item(TTR("Raw"), PROPERTY_NAME_STYLE_RAW);
+	p->add_radio_check_item(TTR("Capitalized"), PROPERTY_NAME_STYLE_CAPITALIZED);
+	p->add_radio_check_item(TTR("Localized"), PROPERTY_NAME_STYLE_LOCALIZED);
+
+	if (!EditorPropertyNameProcessor::is_localization_available()) {
+		const int index = p->get_item_index(PROPERTY_NAME_STYLE_LOCALIZED);
+		p->set_item_disabled(index, true);
+		p->set_item_tooltip(index, TTR("Localization not available for current language."));
 	}
 
+	p->add_separator();
+	p->add_shortcut(ED_SHORTCUT("property_editor/copy_params", TTR("Copy Properties")), OBJECT_COPY_PARAMS);
+	p->add_shortcut(ED_SHORTCUT("property_editor/paste_params", TTR("Paste Properties")), OBJECT_PASTE_PARAMS);
+
 	if (is_resource || is_node) {
 		p->add_separator();
 		p->add_shortcut(ED_SHORTCUT("property_editor/make_subresources_unique", TTR("Make Sub-Resources Unique")), OBJECT_UNIQUE_RESOURCES);
-		p->add_separator();
-		p->add_icon_shortcut(get_icon("HelpSearch", "EditorIcons"), ED_SHORTCUT("property_editor/open_help", TTR("Open in Help")), OBJECT_REQUEST_HELP);
 	}
 
 	List<MethodInfo> methods;
 	p_object->get_method_list(&methods);
 
 	if (!methods.empty()) {
-
 		bool found = false;
 		List<MethodInfo>::Element *I = methods.front();
 		int i = 0;
 		while (I) {
-
 			if (I->get().flags & METHOD_FLAG_EDITOR) {
 				if (!found) {
 					p->add_separator();
@@ -480,13 +511,10 @@ void InspectorDock::update_keying() {
 	bool valid = false;
 
 	if (AnimationPlayerEditor::singleton->get_track_editor()->has_keying()) {
-
 		EditorHistory *editor_history = EditorNode::get_singleton()->get_editor_history();
 		if (editor_history->get_path_size() >= 1) {
-
 			Object *obj = ObjectDB::get_instance(editor_history->get_path_object(0));
 			if (Object::cast_to<Node>(obj)) {
-
 				valid = true;
 			}
 		}
@@ -495,6 +523,10 @@ void InspectorDock::update_keying() {
 	inspector->set_keying(valid);
 }
 
+EditorPropertyNameProcessor::Style InspectorDock::get_property_name_style() const {
+	return property_name_style;
+}
+
 InspectorDock::InspectorDock(EditorNode *p_editor, EditorData &p_editor_data) {
 	set_name("Inspector");
 	set_theme(p_editor->get_gui_base()->get_theme());
@@ -502,6 +534,8 @@ InspectorDock::InspectorDock(EditorNode *p_editor, EditorData &p_editor_data) {
 	editor = p_editor;
 	editor_data = &p_editor_data;
 
+	property_name_style = EditorPropertyNameProcessor::get_default_inspector_style();
+
 	HBoxContainer *general_options_hb = memnew(HBoxContainer);
 	add_child(general_options_hb);
 
@@ -529,6 +563,19 @@ InspectorDock::InspectorDock(EditorNode *p_editor, EditorData &p_editor_data) {
 	resource_save_button->set_focus_mode(Control::FOCUS_NONE);
 	resource_save_button->set_disabled(true);
 
+	resource_extra_button = memnew(MenuButton);
+	resource_extra_button->set_icon(get_icon("GuiTabMenuHl", "EditorIcons"));
+	resource_extra_button->set_tooltip(TTR("Extra resource options."));
+	general_options_hb->add_child(resource_extra_button);
+	resource_extra_button->connect("about_to_show", this, "_prepare_resource_extra_popup");
+	resource_extra_button->get_popup()->add_icon_shortcut(get_icon("ActionPaste", "EditorIcons"), ED_SHORTCUT("property_editor/paste_resource", TTR("Edit Resource from Clipboard")), RESOURCE_EDIT_CLIPBOARD);
+	resource_extra_button->get_popup()->add_icon_shortcut(get_icon("ActionCopy", "EditorIcons"), ED_SHORTCUT("property_editor/copy_resource", TTR("Copy Resource")), RESOURCE_COPY);
+	resource_extra_button->get_popup()->set_item_disabled(1, true);
+	resource_extra_button->get_popup()->add_separator();
+	resource_extra_button->get_popup()->add_shortcut(ED_SHORTCUT("property_editor/unref_resource", TTR("Make Resource Built-In")), RESOURCE_MAKE_BUILT_IN);
+	resource_extra_button->get_popup()->set_item_disabled(3, true);
+	resource_extra_button->get_popup()->connect("id_pressed", this, "_menu_option");
+
 	general_options_hb->add_spacer();
 
 	backward_button = memnew(ToolButton);
@@ -554,35 +601,48 @@ InspectorDock::InspectorDock(EditorNode *p_editor, EditorData &p_editor_data) {
 	history_menu->connect("about_to_show", this, "_prepare_history");
 	history_menu->get_popup()->connect("id_pressed", this, "_select_history");
 
-	HBoxContainer *node_info_hb = memnew(HBoxContainer);
-	add_child(node_info_hb);
-
+	HBoxContainer *subresource_hb = memnew(HBoxContainer);
+	add_child(subresource_hb);
 	editor_path = memnew(EditorPath(editor->get_editor_history()));
 	editor_path->set_h_size_flags(Control::SIZE_EXPAND_FILL);
-	node_info_hb->add_child(editor_path);
+	subresource_hb->add_child(editor_path);
 
-	object_menu = memnew(MenuButton);
-	object_menu->set_icon(get_icon("Tools", "EditorIcons"));
-	node_info_hb->add_child(object_menu);
-	object_menu->set_tooltip(TTR("Object properties."));
-	object_menu->get_popup()->connect("id_pressed", this, "_menu_option");
+	open_docs_button = memnew(Button);
+	open_docs_button->set_flat(true);
+	open_docs_button->set_disabled(true);
+	open_docs_button->set_tooltip(TTR("Open documentation for this object."));
+	open_docs_button->set_icon(get_icon("HelpSearch", "EditorIcons"));
+	open_docs_button->set_shortcut(ED_SHORTCUT("property_editor/open_help", TTR("Open Documentation")));
+	subresource_hb->add_child(open_docs_button);
+	open_docs_button->connect("pressed", this, "_menu_option", varray(OBJECT_REQUEST_HELP));
 
 	new_resource_dialog = memnew(CreateDialog);
 	editor->get_gui_base()->add_child(new_resource_dialog);
 	new_resource_dialog->set_base_type("Resource");
 	new_resource_dialog->connect("create", this, "_resource_created");
 
+	HBoxContainer *property_tools_hb = memnew(HBoxContainer);
+	add_child(property_tools_hb);
+
 	search = memnew(LineEdit);
 	search->set_h_size_flags(Control::SIZE_EXPAND_FILL);
 	search->set_placeholder(TTR("Filter properties"));
 	search->set_right_icon(get_icon("Search", "EditorIcons"));
 	search->set_clear_button_enabled(true);
-	add_child(search);
+	property_tools_hb->add_child(search);
+
+	object_menu = memnew(MenuButton);
+	object_menu->set_icon(get_icon("Tools", "EditorIcons"));
+	property_tools_hb->add_child(object_menu);
+	object_menu->set_tooltip(TTR("Manage object properties."));
+	object_menu->get_popup()->connect("about_to_show", this, "_prepare_menu");
+	object_menu->get_popup()->connect("id_pressed", this, "_menu_option");
 
 	warning = memnew(Button);
 	add_child(warning);
 	warning->set_text(TTR("Changes may be lost!"));
 	warning->set_icon(get_icon("NodeWarning", "EditorIcons"));
+	warning->add_color_override("font_color", get_color("warning_color", "Editor"));
 	warning->set_clip_text(true);
 	warning->hide();
 	warning->connect("pressed", this, "_warning_pressed");
@@ -602,7 +662,7 @@ InspectorDock::InspectorDock(EditorNode *p_editor, EditorData &p_editor_data) {
 	inspector->set_v_size_flags(Control::SIZE_EXPAND_FILL);
 	inspector->set_use_doc_hints(true);
 	inspector->set_hide_script(false);
-	inspector->set_enable_capitalize_paths(bool(EDITOR_GET("interface/inspector/capitalize_properties")));
+	inspector->set_property_name_style(EditorPropertyNameProcessor::get_default_inspector_style());
 	inspector->set_use_folding(!bool(EDITOR_GET("interface/inspector/disable_folding")));
 	inspector->register_text_enter(search);
 	inspector->set_undo_redo(&editor_data->get_undo_redo());
