@@ -5,8 +5,8 @@
 /*                           GODOT ENGINE                                */
 /*                      https://godotengine.org                          */
 /*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
+/* Copyright (c) 2007-2022 Juan Linietsky, Ariel Manzur.                 */
+/* Copyright (c) 2014-2022 Godot Engine contributors (cf. AUTHORS.md).   */
 /*                                                                       */
 /* Permission is hereby granted, free of charge, to any person obtaining */
 /* a copy of this software and associated documentation files (the       */
@@ -34,23 +34,24 @@
 #include "servers/physics_2d_server.h"
 
 void CollisionObject2D::_notification(int p_what) {
-
 	switch (p_what) {
-
 		case NOTIFICATION_ENTER_TREE: {
-
 			Transform2D global_transform = get_global_transform();
 
-			if (area)
+			if (area) {
 				Physics2DServer::get_singleton()->area_set_transform(rid, global_transform);
-			else
+			} else {
 				Physics2DServer::get_singleton()->body_set_state(rid, Physics2DServer::BODY_STATE_TRANSFORM, global_transform);
+			}
 
-			RID space = get_world_2d()->get_space();
+			Ref<World2D> world_ref = get_world_2d();
+			ERR_FAIL_COND(!world_ref.is_valid());
+			RID space = world_ref->get_space();
 			if (area) {
 				Physics2DServer::get_singleton()->area_set_space(rid, space);
-			} else
+			} else {
 				Physics2DServer::get_singleton()->body_set_space(rid, space);
+			}
 
 			_update_pickable();
 
@@ -58,52 +59,108 @@ void CollisionObject2D::_notification(int p_what) {
 		} break;
 
 		case NOTIFICATION_ENTER_CANVAS: {
-
-			if (area)
+			if (area) {
 				Physics2DServer::get_singleton()->area_attach_canvas_instance_id(rid, get_canvas_layer_instance_id());
-			else
+			} else {
 				Physics2DServer::get_singleton()->body_attach_canvas_instance_id(rid, get_canvas_layer_instance_id());
+			}
 		} break;
 
 		case NOTIFICATION_VISIBILITY_CHANGED: {
-
 			_update_pickable();
 		} break;
 		case NOTIFICATION_TRANSFORM_CHANGED: {
-
 			if (only_update_transform_changes) {
 				return;
 			}
 
 			Transform2D global_transform = get_global_transform();
 
-			if (area)
+			if (area) {
 				Physics2DServer::get_singleton()->area_set_transform(rid, global_transform);
-			else
+			} else {
 				Physics2DServer::get_singleton()->body_set_state(rid, Physics2DServer::BODY_STATE_TRANSFORM, global_transform);
+			}
 
 		} break;
 		case NOTIFICATION_EXIT_TREE: {
-
 			if (area) {
 				Physics2DServer::get_singleton()->area_set_space(rid, RID());
-			} else
+			} else {
 				Physics2DServer::get_singleton()->body_set_space(rid, RID());
+			}
 
 		} break;
 
 		case NOTIFICATION_EXIT_CANVAS: {
-
-			if (area)
+			if (area) {
 				Physics2DServer::get_singleton()->area_attach_canvas_instance_id(rid, 0);
-			else
+			} else {
 				Physics2DServer::get_singleton()->body_attach_canvas_instance_id(rid, 0);
+			}
 		} break;
 	}
 }
 
-uint32_t CollisionObject2D::create_shape_owner(Object *p_owner) {
+void CollisionObject2D::set_collision_layer(uint32_t p_layer) {
+	collision_layer = p_layer;
+	if (area) {
+		Physics2DServer::get_singleton()->area_set_collision_layer(get_rid(), p_layer);
+	} else {
+		Physics2DServer::get_singleton()->body_set_collision_layer(get_rid(), p_layer);
+	}
+}
 
+uint32_t CollisionObject2D::get_collision_layer() const {
+	return collision_layer;
+}
+
+void CollisionObject2D::set_collision_mask(uint32_t p_mask) {
+	collision_mask = p_mask;
+	if (area) {
+		Physics2DServer::get_singleton()->area_set_collision_mask(get_rid(), p_mask);
+	} else {
+		Physics2DServer::get_singleton()->body_set_collision_mask(get_rid(), p_mask);
+	}
+}
+
+uint32_t CollisionObject2D::get_collision_mask() const {
+	return collision_mask;
+}
+
+void CollisionObject2D::set_collision_layer_bit(int p_bit, bool p_value) {
+	ERR_FAIL_INDEX_MSG(p_bit, 32, "Collision layer bit must be between 0 and 31 inclusive.");
+	uint32_t collision_layer = get_collision_layer();
+	if (p_value) {
+		collision_layer |= 1 << p_bit;
+	} else {
+		collision_layer &= ~(1 << p_bit);
+	}
+	set_collision_layer(collision_layer);
+}
+
+bool CollisionObject2D::get_collision_layer_bit(int p_bit) const {
+	ERR_FAIL_INDEX_V_MSG(p_bit, 32, false, "Collision layer bit must be between 0 and 31 inclusive.");
+	return get_collision_layer() & (1 << p_bit);
+}
+
+void CollisionObject2D::set_collision_mask_bit(int p_bit, bool p_value) {
+	ERR_FAIL_INDEX_MSG(p_bit, 32, "Collision mask bit must be between 0 and 31 inclusive.");
+	uint32_t mask = get_collision_mask();
+	if (p_value) {
+		mask |= 1 << p_bit;
+	} else {
+		mask &= ~(1 << p_bit);
+	}
+	set_collision_mask(mask);
+}
+
+bool CollisionObject2D::get_collision_mask_bit(int p_bit) const {
+	ERR_FAIL_INDEX_V_MSG(p_bit, 32, false, "Collision mask bit must be between 0 and 31 inclusive.");
+	return get_collision_mask() & (1 << p_bit);
+}
+
+uint32_t CollisionObject2D::create_shape_owner(Object *p_owner) {
 	ShapeData sd;
 	uint32_t id;
 
@@ -113,7 +170,7 @@ uint32_t CollisionObject2D::create_shape_owner(Object *p_owner) {
 		id = shapes.back()->key() + 1;
 	}
 
-	sd.owner = p_owner;
+	sd.owner_id = p_owner ? p_owner->get_instance_id() : 0;
 
 	shapes[id] = sd;
 
@@ -121,7 +178,6 @@ uint32_t CollisionObject2D::create_shape_owner(Object *p_owner) {
 }
 
 void CollisionObject2D::remove_shape_owner(uint32_t owner) {
-
 	ERR_FAIL_COND(!shapes.has(owner));
 
 	shape_owner_clear_shapes(owner);
@@ -144,16 +200,15 @@ void CollisionObject2D::shape_owner_set_disabled(uint32_t p_owner, bool p_disabl
 }
 
 bool CollisionObject2D::is_shape_owner_disabled(uint32_t p_owner) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), false);
 
 	return shapes[p_owner].disabled;
 }
 
 void CollisionObject2D::shape_owner_set_one_way_collision(uint32_t p_owner, bool p_enable) {
-
-	if (area)
+	if (area) {
 		return; //not for areas
+	}
 
 	ERR_FAIL_COND(!shapes.has(p_owner));
 
@@ -165,16 +220,15 @@ void CollisionObject2D::shape_owner_set_one_way_collision(uint32_t p_owner, bool
 }
 
 bool CollisionObject2D::is_shape_owner_one_way_collision_enabled(uint32_t p_owner) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), false);
 
 	return shapes[p_owner].one_way_collision;
 }
 
 void CollisionObject2D::shape_owner_set_one_way_collision_margin(uint32_t p_owner, float p_margin) {
-
-	if (area)
+	if (area) {
 		return; //not for areas
+	}
 
 	ERR_FAIL_COND(!shapes.has(p_owner));
 
@@ -186,21 +240,18 @@ void CollisionObject2D::shape_owner_set_one_way_collision_margin(uint32_t p_owne
 }
 
 float CollisionObject2D::get_shape_owner_one_way_collision_margin(uint32_t p_owner) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), 0);
 
 	return shapes[p_owner].one_way_collision_margin;
 }
 
 void CollisionObject2D::get_shape_owners(List<uint32_t> *r_owners) {
-
 	for (Map<uint32_t, ShapeData>::Element *E = shapes.front(); E; E = E->next()) {
 		r_owners->push_back(E->key());
 	}
 }
 
 Array CollisionObject2D::_get_shape_owners() {
-
 	Array ret;
 	for (Map<uint32_t, ShapeData>::Element *E = shapes.front(); E; E = E->next()) {
 		ret.push_back(E->key());
@@ -210,7 +261,6 @@ Array CollisionObject2D::_get_shape_owners() {
 }
 
 void CollisionObject2D::shape_owner_set_transform(uint32_t p_owner, const Transform2D &p_transform) {
-
 	ERR_FAIL_COND(!shapes.has(p_owner));
 
 	ShapeData &sd = shapes[p_owner];
@@ -225,21 +275,18 @@ void CollisionObject2D::shape_owner_set_transform(uint32_t p_owner, const Transf
 	}
 }
 Transform2D CollisionObject2D::shape_owner_get_transform(uint32_t p_owner) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), Transform2D());
 
 	return shapes[p_owner].xform;
 }
 
 Object *CollisionObject2D::shape_owner_get_owner(uint32_t p_owner) const {
+	ERR_FAIL_COND_V(!shapes.has(p_owner), nullptr);
 
-	ERR_FAIL_COND_V(!shapes.has(p_owner), NULL);
-
-	return shapes[p_owner].owner;
+	return ObjectDB::get_instance(shapes[p_owner].owner_id);
 }
 
 void CollisionObject2D::shape_owner_add_shape(uint32_t p_owner, const Ref<Shape2D> &p_shape) {
-
 	ERR_FAIL_COND(!shapes.has(p_owner));
 	ERR_FAIL_COND(p_shape.is_null());
 
@@ -257,20 +304,17 @@ void CollisionObject2D::shape_owner_add_shape(uint32_t p_owner, const Ref<Shape2
 	total_subshapes++;
 }
 int CollisionObject2D::shape_owner_get_shape_count(uint32_t p_owner) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), 0);
 
 	return shapes[p_owner].shapes.size();
 }
 Ref<Shape2D> CollisionObject2D::shape_owner_get_shape(uint32_t p_owner, int p_shape) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), Ref<Shape2D>());
 	ERR_FAIL_INDEX_V(p_shape, shapes[p_owner].shapes.size(), Ref<Shape2D>());
 
 	return shapes[p_owner].shapes[p_shape].shape;
 }
 int CollisionObject2D::shape_owner_get_shape_index(uint32_t p_owner, int p_shape) const {
-
 	ERR_FAIL_COND_V(!shapes.has(p_owner), -1);
 	ERR_FAIL_INDEX_V(p_shape, shapes[p_owner].shapes.size(), -1);
 
@@ -278,7 +322,6 @@ int CollisionObject2D::shape_owner_get_shape_index(uint32_t p_owner, int p_shape
 }
 
 void CollisionObject2D::shape_owner_remove_shape(uint32_t p_owner, int p_shape) {
-
 	ERR_FAIL_COND(!shapes.has(p_owner));
 	ERR_FAIL_INDEX(p_shape, shapes[p_owner].shapes.size());
 
@@ -303,7 +346,6 @@ void CollisionObject2D::shape_owner_remove_shape(uint32_t p_owner, int p_shape)
 }
 
 void CollisionObject2D::shape_owner_clear_shapes(uint32_t p_owner) {
-
 	ERR_FAIL_COND(!shapes.has(p_owner));
 
 	while (shape_owner_get_shape_count(p_owner) > 0) {
@@ -312,8 +354,7 @@ void CollisionObject2D::shape_owner_clear_shapes(uint32_t p_owner) {
 }
 
 uint32_t CollisionObject2D::shape_find_owner(int p_shape_index) const {
-
-	ERR_FAIL_INDEX_V(p_shape_index, total_subshapes, 0);
+	ERR_FAIL_INDEX_V(p_shape_index, total_subshapes, UINT32_MAX);
 
 	for (const Map<uint32_t, ShapeData>::Element *E = shapes.front(); E; E = E->next()) {
 		for (int i = 0; i < E->get().shapes.size(); i++) {
@@ -324,25 +365,23 @@ uint32_t CollisionObject2D::shape_find_owner(int p_shape_index) const {
 	}
 
 	//in theory it should be unreachable
-	return 0;
+	ERR_FAIL_V_MSG(UINT32_MAX, "Can't find owner for shape index " + itos(p_shape_index) + ".");
 }
 
 void CollisionObject2D::set_pickable(bool p_enabled) {
-
-	if (pickable == p_enabled)
+	if (pickable == p_enabled) {
 		return;
+	}
 
 	pickable = p_enabled;
 	_update_pickable();
 }
 
 bool CollisionObject2D::is_pickable() const {
-
 	return pickable;
 }
 
 void CollisionObject2D::_input_event(Node *p_viewport, const Ref<InputEvent> &p_input_event, int p_shape) {
-
 	if (get_script_instance()) {
 		get_script_instance()->call(SceneStringNames::get_singleton()->_input_event, p_viewport, p_input_event, p_shape);
 	}
@@ -350,7 +389,6 @@ void CollisionObject2D::_input_event(Node *p_viewport, const Ref<InputEvent> &p_
 }
 
 void CollisionObject2D::_mouse_enter() {
-
 	if (get_script_instance()) {
 		get_script_instance()->call(SceneStringNames::get_singleton()->_mouse_enter);
 	}
@@ -358,7 +396,6 @@ void CollisionObject2D::_mouse_enter() {
 }
 
 void CollisionObject2D::_mouse_exit() {
-
 	if (get_script_instance()) {
 		get_script_instance()->call(SceneStringNames::get_singleton()->_mouse_exit);
 	}
@@ -370,18 +407,19 @@ void CollisionObject2D::set_only_update_transform_changes(bool p_enable) {
 }
 
 void CollisionObject2D::_update_pickable() {
-	if (!is_inside_tree())
+	if (!is_inside_tree()) {
 		return;
+	}
 
 	bool is_pickable = pickable && is_visible_in_tree();
-	if (area)
+	if (area) {
 		Physics2DServer::get_singleton()->area_set_pickable(rid, is_pickable);
-	else
+	} else {
 		Physics2DServer::get_singleton()->body_set_pickable(rid, is_pickable);
+	}
 }
 
 String CollisionObject2D::get_configuration_warning() const {
-
 	String warning = Node2D::get_configuration_warning();
 
 	if (shapes.empty()) {
@@ -395,9 +433,15 @@ String CollisionObject2D::get_configuration_warning() const {
 }
 
 void CollisionObject2D::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("get_rid"), &CollisionObject2D::get_rid);
-
+	ClassDB::bind_method(D_METHOD("set_collision_layer", "layer"), &CollisionObject2D::set_collision_layer);
+	ClassDB::bind_method(D_METHOD("get_collision_layer"), &CollisionObject2D::get_collision_layer);
+	ClassDB::bind_method(D_METHOD("set_collision_mask", "mask"), &CollisionObject2D::set_collision_mask);
+	ClassDB::bind_method(D_METHOD("get_collision_mask"), &CollisionObject2D::get_collision_mask);
+	ClassDB::bind_method(D_METHOD("set_collision_layer_bit", "bit", "value"), &CollisionObject2D::set_collision_layer_bit);
+	ClassDB::bind_method(D_METHOD("get_collision_layer_bit", "bit"), &CollisionObject2D::get_collision_layer_bit);
+	ClassDB::bind_method(D_METHOD("set_collision_mask_bit", "bit", "value"), &CollisionObject2D::set_collision_mask_bit);
+	ClassDB::bind_method(D_METHOD("get_collision_mask_bit", "bit"), &CollisionObject2D::get_collision_mask_bit);
 	ClassDB::bind_method(D_METHOD("set_pickable", "enabled"), &CollisionObject2D::set_pickable);
 	ClassDB::bind_method(D_METHOD("is_pickable"), &CollisionObject2D::is_pickable);
 	ClassDB::bind_method(D_METHOD("create_shape_owner", "owner"), &CollisionObject2D::create_shape_owner);
@@ -426,13 +470,15 @@ void CollisionObject2D::_bind_methods() {
 	ADD_SIGNAL(MethodInfo("mouse_entered"));
 	ADD_SIGNAL(MethodInfo("mouse_exited"));
 
-	ADD_GROUP("Pickable", "input_");
+	ADD_GROUP("Collision", "collision_");
+	ADD_PROPERTY(PropertyInfo(Variant::INT, "collision_layer", PROPERTY_HINT_LAYERS_2D_PHYSICS), "set_collision_layer", "get_collision_layer");
+	ADD_PROPERTY(PropertyInfo(Variant::INT, "collision_mask", PROPERTY_HINT_LAYERS_2D_PHYSICS), "set_collision_mask", "get_collision_mask");
+
+	ADD_GROUP("Input", "input_");
 	ADD_PROPERTY(PropertyInfo(Variant::BOOL, "input_pickable"), "set_pickable", "is_pickable");
-	ADD_GROUP("", "");
 }
 
 CollisionObject2D::CollisionObject2D(RID p_rid, bool p_area) {
-
 	rid = p_rid;
 	area = p_area;
 	pickable = true;
@@ -441,7 +487,6 @@ CollisionObject2D::CollisionObject2D(RID p_rid, bool p_area) {
 	only_update_transform_changes = false;
 
 	if (p_area) {
-
 		Physics2DServer::get_singleton()->area_attach_object_instance_id(rid, get_instance_id());
 	} else {
 		Physics2DServer::get_singleton()->body_attach_object_instance_id(rid, get_instance_id());
@@ -449,13 +494,11 @@ CollisionObject2D::CollisionObject2D(RID p_rid, bool p_area) {
 }
 
 CollisionObject2D::CollisionObject2D() {
-
 	//owner=
 
 	set_notify_transform(true);
 }
 
 CollisionObject2D::~CollisionObject2D() {
-
 	Physics2DServer::get_singleton()->free(rid);
 }
