@@ -5,8 +5,8 @@
 /*                           GODOT ENGINE                                */
 /*                      https://godotengine.org                          */
 /*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
+/* Copyright (c) 2007-2022 Juan Linietsky, Ariel Manzur.                 */
+/* Copyright (c) 2014-2022 Godot Engine contributors (cf. AUTHORS.md).   */
 /*                                                                       */
 /* Permission is hereby granted, free of charge, to any person obtaining */
 /* a copy of this software and associated documentation files (the       */
@@ -31,8 +31,9 @@
 #include "option_button.h"
 #include "core/print_string.h"
 
-Size2 OptionButton::get_minimum_size() const {
+static const int NONE_SELECTED = -1;
 
+Size2 OptionButton::get_minimum_size() const {
 	Size2 minsize = Button::get_minimum_size();
 
 	if (has_icon("arrow")) {
@@ -50,12 +51,11 @@ Size2 OptionButton::get_minimum_size() const {
 }
 
 void OptionButton::_notification(int p_what) {
-
 	switch (p_what) {
 		case NOTIFICATION_DRAW: {
-
-			if (!has_icon("arrow"))
+			if (!has_icon("arrow")) {
 				return;
+			}
 
 			RID ci = get_canvas_item();
 			Ref<Texture> arrow = Control::get_icon("arrow");
@@ -72,7 +72,11 @@ void OptionButton::_notification(int p_what) {
 						clr = get_color("font_color_disabled");
 						break;
 					default:
-						clr = get_color("font_color");
+						if (has_focus()) {
+							clr = get_color("font_color_focus");
+						} else {
+							clr = get_color("font_color");
+						}
 				}
 			}
 
@@ -82,13 +86,11 @@ void OptionButton::_notification(int p_what) {
 			arrow->draw(ci, ofs, clr);
 		} break;
 		case NOTIFICATION_THEME_CHANGED: {
-
 			if (has_icon("arrow")) {
 				_set_internal_margin(MARGIN_RIGHT, Control::get_icon("arrow")->get_width());
 			}
 		} break;
 		case NOTIFICATION_VISIBILITY_CHANGED: {
-
 			if (!is_visible_in_tree()) {
 				popup->hide();
 			}
@@ -101,177 +103,192 @@ void OptionButton::_focused(int p_which) {
 }
 
 void OptionButton::_selected(int p_which) {
-
 	_select(p_which, true);
 }
 
 void OptionButton::pressed() {
-
 	Size2 size = get_size();
 	popup->set_global_position(get_global_position() + Size2(0, size.height * get_global_transform().get_scale().y));
 	popup->set_size(Size2(size.width, 0));
 	popup->set_scale(get_global_transform().get_scale());
+
+	// If not triggered by the mouse, start the popup with the checked item (or the first enabled one) focused.
+	if (!_was_pressed_by_mouse()) {
+		if (current > -1 && !popup->is_item_disabled(current)) {
+			popup->set_current_index(current);
+		} else {
+			for (int i = 0; i < popup->get_item_count(); i++) {
+				if (!popup->is_item_disabled(i)) {
+					popup->set_current_index(i);
+					break;
+				}
+			}
+		}
+	}
+
 	popup->popup();
 }
 
 void OptionButton::add_icon_item(const Ref<Texture> &p_icon, const String &p_label, int p_id) {
-
 	popup->add_icon_radio_check_item(p_icon, p_label, p_id);
-	if (popup->get_item_count() == 1)
+	if (popup->get_item_count() == 1) {
 		select(0);
+	}
 }
 void OptionButton::add_item(const String &p_label, int p_id) {
-
 	popup->add_radio_check_item(p_label, p_id);
-	if (popup->get_item_count() == 1)
+	if (popup->get_item_count() == 1) {
 		select(0);
+	}
 }
 
 void OptionButton::set_item_text(int p_idx, const String &p_text) {
-
 	popup->set_item_text(p_idx, p_text);
 
-	if (current == p_idx)
+	if (current == p_idx) {
 		set_text(p_text);
+	}
 }
 void OptionButton::set_item_icon(int p_idx, const Ref<Texture> &p_icon) {
-
 	popup->set_item_icon(p_idx, p_icon);
 
-	if (current == p_idx)
+	if (current == p_idx) {
 		set_icon(p_icon);
+	}
 }
 void OptionButton::set_item_id(int p_idx, int p_id) {
-
 	popup->set_item_id(p_idx, p_id);
 }
 
 void OptionButton::set_item_metadata(int p_idx, const Variant &p_metadata) {
-
 	popup->set_item_metadata(p_idx, p_metadata);
 }
 
-void OptionButton::set_item_disabled(int p_idx, bool p_disabled) {
+void OptionButton::set_item_tooltip(int p_idx, const String &p_tooltip) {
+	popup->set_item_tooltip(p_idx, p_tooltip);
+}
 
+void OptionButton::set_item_disabled(int p_idx, bool p_disabled) {
 	popup->set_item_disabled(p_idx, p_disabled);
 }
 
 String OptionButton::get_item_text(int p_idx) const {
-
 	return popup->get_item_text(p_idx);
 }
 
 Ref<Texture> OptionButton::get_item_icon(int p_idx) const {
-
 	return popup->get_item_icon(p_idx);
 }
 
 int OptionButton::get_item_id(int p_idx) const {
+	if (p_idx == NONE_SELECTED) {
+		return NONE_SELECTED;
+	}
 
 	return popup->get_item_id(p_idx);
 }
 
 int OptionButton::get_item_index(int p_id) const {
-
 	return popup->get_item_index(p_id);
 }
 
 Variant OptionButton::get_item_metadata(int p_idx) const {
-
 	return popup->get_item_metadata(p_idx);
 }
 
-bool OptionButton::is_item_disabled(int p_idx) const {
+String OptionButton::get_item_tooltip(int p_idx) const {
+	return popup->get_item_tooltip(p_idx);
+}
 
+bool OptionButton::is_item_disabled(int p_idx) const {
 	return popup->is_item_disabled(p_idx);
 }
 
 int OptionButton::get_item_count() const {
-
 	return popup->get_item_count();
 }
 
 void OptionButton::add_separator() {
-
 	popup->add_separator();
 }
 
 void OptionButton::clear() {
-
 	popup->clear();
 	set_text("");
-	current = -1;
+	current = NONE_SELECTED;
 }
 
 void OptionButton::_select(int p_which, bool p_emit) {
-
-	if (p_which < 0)
+	if (p_which == current) {
 		return;
-	if (p_which == current)
-		return;
-
-	ERR_FAIL_INDEX(p_which, popup->get_item_count());
-
-	for (int i = 0; i < popup->get_item_count(); i++) {
-
-		popup->set_item_checked(i, i == p_which);
 	}
 
-	current = p_which;
-	set_text(popup->get_item_text(current));
-	set_icon(popup->get_item_icon(current));
+	if (p_which == NONE_SELECTED) {
+		for (int i = 0; i < popup->get_item_count(); i++) {
+			popup->set_item_checked(i, false);
+		}
 
-	if (is_inside_tree() && p_emit)
+		current = NONE_SELECTED;
+		set_text("");
+		set_icon(NULL);
+	} else {
+		ERR_FAIL_INDEX(p_which, popup->get_item_count());
+
+		for (int i = 0; i < popup->get_item_count(); i++) {
+			popup->set_item_checked(i, i == p_which);
+		}
+
+		current = p_which;
+		set_text(popup->get_item_text(current));
+		set_icon(popup->get_item_icon(current));
+	}
+
+	if (is_inside_tree() && p_emit) {
 		emit_signal("item_selected", current);
+	}
 }
 
 void OptionButton::_select_int(int p_which) {
-
-	if (p_which < 0 || p_which >= popup->get_item_count())
+	if (p_which < NONE_SELECTED || p_which >= popup->get_item_count()) {
 		return;
+	}
 	_select(p_which, false);
 }
 
 void OptionButton::select(int p_idx) {
-
 	_select(p_idx, false);
 }
 
 int OptionButton::get_selected() const {
-
 	return current;
 }
 
 int OptionButton::get_selected_id() const {
-
-	int idx = get_selected();
-	if (idx < 0)
-		return 0;
 	return get_item_id(current);
 }
-Variant OptionButton::get_selected_metadata() const {
 
+Variant OptionButton::get_selected_metadata() const {
 	int idx = get_selected();
-	if (idx < 0)
+	if (idx < 0) {
 		return Variant();
+	}
 	return get_item_metadata(current);
 }
 
 void OptionButton::remove_item(int p_idx) {
-
 	popup->remove_item(p_idx);
+	if (current == p_idx) {
+		_select(NONE_SELECTED);
+	}
 }
 
 PopupMenu *OptionButton::get_popup() const {
-
 	return popup;
 }
 
 Array OptionButton::_get_items() const {
-
 	Array items;
 	for (int i = 0; i < get_item_count(); i++) {
-
 		items.push_back(get_item_text(i));
 		items.push_back(get_item_icon(i));
 		items.push_back(is_item_disabled(i));
@@ -282,12 +299,10 @@ Array OptionButton::_get_items() const {
 	return items;
 }
 void OptionButton::_set_items(const Array &p_items) {
-
 	ERR_FAIL_COND(p_items.size() % 5);
 	clear();
 
 	for (int i = 0; i < p_items.size(); i += 5) {
-
 		String text = p_items[i + 0];
 		Ref<Texture> icon = p_items[i + 1];
 		bool disabled = p_items[i + 2];
@@ -303,12 +318,10 @@ void OptionButton::_set_items(const Array &p_items) {
 }
 
 void OptionButton::get_translatable_strings(List<String> *p_strings) const {
-
 	popup->get_translatable_strings(p_strings);
 }
 
 void OptionButton::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_selected"), &OptionButton::_selected);
 	ClassDB::bind_method(D_METHOD("_focused"), &OptionButton::_focused);
 
@@ -319,11 +332,13 @@ void OptionButton::_bind_methods() {
 	ClassDB::bind_method(D_METHOD("set_item_disabled", "idx", "disabled"), &OptionButton::set_item_disabled);
 	ClassDB::bind_method(D_METHOD("set_item_id", "idx", "id"), &OptionButton::set_item_id);
 	ClassDB::bind_method(D_METHOD("set_item_metadata", "idx", "metadata"), &OptionButton::set_item_metadata);
+	ClassDB::bind_method(D_METHOD("set_item_tooltip", "idx", "tooltip"), &OptionButton::set_item_tooltip);
 	ClassDB::bind_method(D_METHOD("get_item_text", "idx"), &OptionButton::get_item_text);
 	ClassDB::bind_method(D_METHOD("get_item_icon", "idx"), &OptionButton::get_item_icon);
 	ClassDB::bind_method(D_METHOD("get_item_id", "idx"), &OptionButton::get_item_id);
 	ClassDB::bind_method(D_METHOD("get_item_index", "id"), &OptionButton::get_item_index);
 	ClassDB::bind_method(D_METHOD("get_item_metadata", "idx"), &OptionButton::get_item_metadata);
+	ClassDB::bind_method(D_METHOD("get_item_tooltip", "idx"), &OptionButton::get_item_tooltip);
 	ClassDB::bind_method(D_METHOD("is_item_disabled", "idx"), &OptionButton::is_item_disabled);
 	ClassDB::bind_method(D_METHOD("get_item_count"), &OptionButton::get_item_count);
 	ClassDB::bind_method(D_METHOD("add_separator"), &OptionButton::add_separator);
@@ -348,7 +363,6 @@ void OptionButton::_bind_methods() {
 }
 
 OptionButton::OptionButton() {
-
 	current = -1;
 	set_toggle_mode(true);
 	set_text_align(ALIGN_LEFT);
