@@ -5,8 +5,8 @@
 /*                           GODOT ENGINE                                */
 /*                      https://godotengine.org                          */
 /*************************************************************************/
-/* Copyright (c) 2007-2020 Juan Linietsky, Ariel Manzur.                 */
-/* Copyright (c) 2014-2020 Godot Engine contributors (cf. AUTHORS.md).   */
+/* Copyright (c) 2007-2022 Juan Linietsky, Ariel Manzur.                 */
+/* Copyright (c) 2014-2022 Godot Engine contributors (cf. AUTHORS.md).   */
 /*                                                                       */
 /* Permission is hereby granted, free of charge, to any person obtaining */
 /* a copy of this software and associated documentation files (the       */
@@ -37,11 +37,11 @@
 bool ScrollBar::focus_by_default = false;
 
 void ScrollBar::set_can_focus_by_default(bool p_can_focus) {
-
 	focus_by_default = p_can_focus;
 }
 
 void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
+	ERR_FAIL_COND(p_event.is_null());
 
 	Ref<InputEventMouseMotion> m = p_event;
 	if (!m.is_valid() || drag.active) {
@@ -54,22 +54,20 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 		accept_event();
 
 		if (b->get_button_index() == BUTTON_WHEEL_DOWN && b->is_pressed()) {
-
 			set_value(get_value() + get_page() / 4.0);
 			accept_event();
 		}
 
 		if (b->get_button_index() == BUTTON_WHEEL_UP && b->is_pressed()) {
-
 			set_value(get_value() - get_page() / 4.0);
 			accept_event();
 		}
 
-		if (b->get_button_index() != BUTTON_LEFT)
+		if (b->get_button_index() != BUTTON_LEFT) {
 			return;
+		}
 
 		if (b->is_pressed()) {
-
 			double ofs = orientation == VERTICAL ? b->get_position().y : b->get_position().x;
 			Ref<Texture> decr = get_icon("decrement");
 			Ref<Texture> incr = get_icon("increment");
@@ -81,21 +79,22 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 			double total = orientation == VERTICAL ? get_size().height : get_size().width;
 
 			if (ofs < decr_size) {
-
+				decr_active = true;
 				set_value(get_value() - (custom_step >= 0 ? custom_step : get_step()));
+				update();
 				return;
 			}
 
 			if (ofs > total - incr_size) {
-
+				incr_active = true;
 				set_value(get_value() + (custom_step >= 0 ? custom_step : get_step()));
+				update();
 				return;
 			}
 
 			ofs -= decr_size;
 
 			if (ofs < grabber_ofs) {
-
 				if (scrolling) {
 					target_scroll = CLAMP(target_scroll - get_page(), get_min(), get_max() - get_page());
 				} else {
@@ -114,7 +113,6 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 			ofs -= grabber_ofs;
 
 			if (ofs < grabber_size) {
-
 				drag.active = true;
 				drag.pos_at_click = grabber_ofs + ofs;
 				drag.value_at_click = get_as_ratio();
@@ -135,18 +133,17 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 			}
 
 		} else {
-
+			incr_active = false;
+			decr_active = false;
 			drag.active = false;
 			update();
 		}
 	}
 
 	if (m.is_valid()) {
-
 		accept_event();
 
 		if (drag.active) {
-
 			double ofs = orientation == VERTICAL ? m->get_position().y : m->get_position().x;
 			Ref<Texture> decr = get_icon("decrement");
 
@@ -157,7 +154,6 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 
 			set_as_ratio(drag.value_at_click + diff);
 		} else {
-
 			double ofs = orientation == VERTICAL ? m->get_position().y : m->get_position().x;
 			Ref<Texture> decr = get_icon("decrement");
 			Ref<Texture> incr = get_icon("increment");
@@ -169,20 +165,16 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 			HighlightStatus new_hilite;
 
 			if (ofs < decr_size) {
-
 				new_hilite = HIGHLIGHT_DECR;
 
 			} else if (ofs > total - incr_size) {
-
 				new_hilite = HIGHLIGHT_INCR;
 
 			} else {
-
 				new_hilite = HIGHLIGHT_RANGE;
 			}
 
 			if (new_hilite != highlight) {
-
 				highlight = new_hilite;
 				update();
 			}
@@ -190,95 +182,108 @@ void ScrollBar::_gui_input(Ref<InputEvent> p_event) {
 	}
 
 	if (p_event->is_pressed()) {
-
 		if (p_event->is_action("ui_left")) {
-
-			if (orientation != HORIZONTAL)
+			if (orientation != HORIZONTAL) {
 				return;
+			}
 			set_value(get_value() - (custom_step >= 0 ? custom_step : get_step()));
 
 		} else if (p_event->is_action("ui_right")) {
-
-			if (orientation != HORIZONTAL)
+			if (orientation != HORIZONTAL) {
 				return;
+			}
 			set_value(get_value() + (custom_step >= 0 ? custom_step : get_step()));
 
 		} else if (p_event->is_action("ui_up")) {
-
-			if (orientation != VERTICAL)
+			if (orientation != VERTICAL) {
 				return;
+			}
 
 			set_value(get_value() - (custom_step >= 0 ? custom_step : get_step()));
 
 		} else if (p_event->is_action("ui_down")) {
-
-			if (orientation != VERTICAL)
+			if (orientation != VERTICAL) {
 				return;
+			}
 			set_value(get_value() + (custom_step >= 0 ? custom_step : get_step()));
 
 		} else if (p_event->is_action("ui_home")) {
-
 			set_value(get_min());
 
 		} else if (p_event->is_action("ui_end")) {
-
 			set_value(get_max());
 		}
 	}
 }
 
 void ScrollBar::_notification(int p_what) {
-
 	if (p_what == NOTIFICATION_DRAW) {
-
 		RID ci = get_canvas_item();
 
-		Ref<Texture> decr = highlight == HIGHLIGHT_DECR ? get_icon("decrement_highlight") : get_icon("decrement");
-		Ref<Texture> incr = highlight == HIGHLIGHT_INCR ? get_icon("increment_highlight") : get_icon("increment");
+		Ref<Texture> decr, incr;
+
+		if (decr_active) {
+			decr = get_icon("decrement_pressed");
+		} else if (highlight == HIGHLIGHT_DECR) {
+			decr = get_icon("decrement_highlight");
+		} else {
+			decr = get_icon("decrement");
+		}
+
+		if (incr_active) {
+			incr = get_icon("increment_pressed");
+		} else if (highlight == HIGHLIGHT_INCR) {
+			incr = get_icon("increment_highlight");
+		} else {
+			incr = get_icon("increment");
+		}
+
 		Ref<StyleBox> bg = has_focus() ? get_stylebox("scroll_focus") : get_stylebox("scroll");
 
 		Ref<StyleBox> grabber;
-		if (drag.active)
+		if (drag.active) {
 			grabber = get_stylebox("grabber_pressed");
-		else if (highlight == HIGHLIGHT_RANGE)
+		} else if (highlight == HIGHLIGHT_RANGE) {
 			grabber = get_stylebox("grabber_highlight");
-		else
+		} else {
 			grabber = get_stylebox("grabber");
+		}
 
 		Point2 ofs;
 
 		decr->draw(ci, Point2());
 
-		if (orientation == HORIZONTAL)
+		if (orientation == HORIZONTAL) {
 			ofs.x += decr->get_width();
-		else
+		} else {
 			ofs.y += decr->get_height();
+		}
 
 		Size2 area = get_size();
 
-		if (orientation == HORIZONTAL)
+		if (orientation == HORIZONTAL) {
 			area.width -= incr->get_width() + decr->get_width();
-		else
+		} else {
 			area.height -= incr->get_height() + decr->get_height();
+		}
 
 		bg->draw(ci, Rect2(ofs, area));
 
-		if (orientation == HORIZONTAL)
+		if (orientation == HORIZONTAL) {
 			ofs.width += area.width;
-		else
+		} else {
 			ofs.height += area.height;
+		}
 
 		incr->draw(ci, ofs);
 		Rect2 grabber_rect;
 
 		if (orientation == HORIZONTAL) {
-
 			grabber_rect.size.width = get_grabber_size();
 			grabber_rect.size.height = get_size().height;
 			grabber_rect.position.y = 0;
 			grabber_rect.position.x = get_grabber_offset() + decr->get_width() + bg->get_margin(MARGIN_LEFT);
 		} else {
-
 			grabber_rect.size.width = get_size().width;
 			grabber_rect.size.height = get_grabber_size();
 			grabber_rect.position.y = get_grabber_offset() + decr->get_height() + bg->get_margin(MARGIN_TOP);
@@ -289,7 +294,6 @@ void ScrollBar::_notification(int p_what) {
 	}
 
 	if (p_what == NOTIFICATION_ENTER_TREE) {
-
 		if (has_node(drag_node_path)) {
 			Node *n = get_node(drag_node_path);
 			drag_node = Object::cast_to<Control>(n);
@@ -301,17 +305,15 @@ void ScrollBar::_notification(int p_what) {
 		}
 	}
 	if (p_what == NOTIFICATION_EXIT_TREE) {
-
 		if (drag_node) {
 			drag_node->disconnect("gui_input", this, "_drag_node_input");
 			drag_node->disconnect("tree_exiting", this, "_drag_node_exit");
 		}
 
-		drag_node = NULL;
+		drag_node = nullptr;
 	}
 
 	if (p_what == NOTIFICATION_INTERNAL_PHYSICS_PROCESS) {
-
 		if (scrolling) {
 			if (get_value() != target_scroll) {
 				double target = target_scroll - get_value();
@@ -331,16 +333,13 @@ void ScrollBar::_notification(int p_what) {
 			}
 
 		} else if (drag_node_touching) {
-
 			if (drag_node_touching_deaccel) {
-
 				Vector2 pos = Vector2(orientation == HORIZONTAL ? get_value() : 0, orientation == VERTICAL ? get_value() : 0);
 				pos += drag_node_speed * get_physics_process_delta_time();
 
 				bool turnoff = false;
 
 				if (orientation == HORIZONTAL) {
-
 					if (pos.x < 0) {
 						pos.x = 0;
 						turnoff = true;
@@ -364,7 +363,6 @@ void ScrollBar::_notification(int p_what) {
 					drag_node_speed.x = sgn_x * val_x;
 
 				} else {
-
 					if (pos.y < 0) {
 						pos.y = 0;
 						turnoff = true;
@@ -394,9 +392,7 @@ void ScrollBar::_notification(int p_what) {
 				}
 
 			} else {
-
 				if (time_since_motion == 0 || time_since_motion > 0.1) {
-
 					Vector2 diff = drag_node_accum - last_drag_node_accum;
 					last_drag_node_accum = drag_node_accum;
 					drag_node_speed = diff / get_physics_process_delta_time();
@@ -408,24 +404,22 @@ void ScrollBar::_notification(int p_what) {
 	}
 
 	if (p_what == NOTIFICATION_MOUSE_EXIT) {
-
 		highlight = HIGHLIGHT_NONE;
 		update();
 	}
 }
 
 double ScrollBar::get_grabber_min_size() const {
-
 	Ref<StyleBox> grabber = get_stylebox("grabber");
 	Size2 gminsize = grabber->get_minimum_size() + grabber->get_center_size();
 	return (orientation == VERTICAL) ? gminsize.height : gminsize.width;
 }
 
 double ScrollBar::get_grabber_size() const {
-
 	float range = get_max() - get_min();
-	if (range <= 0)
+	if (range <= 0) {
 		return 0;
+	}
 
 	float page = (get_page() > 0) ? get_page() : 0;
 	/*
@@ -463,17 +457,14 @@ double ScrollBar::get_area_size() const {
 }
 
 double ScrollBar::get_area_offset() const {
-
 	double ofs = 0;
 
 	if (orientation == VERTICAL) {
-
 		ofs += get_stylebox("hscroll")->get_margin(MARGIN_TOP);
 		ofs += get_icon("decrement")->get_height();
 	}
 
 	if (orientation == HORIZONTAL) {
-
 		ofs += get_stylebox("hscroll")->get_margin(MARGIN_LEFT);
 		ofs += get_icon("decrement")->get_width();
 	}
@@ -482,31 +473,28 @@ double ScrollBar::get_area_offset() const {
 }
 
 double ScrollBar::get_click_pos(const Point2 &p_pos) const {
-
 	float pos = (orientation == VERTICAL) ? p_pos.y : p_pos.x;
 	pos -= get_area_offset();
 
 	float area = get_area_size();
-	if (area == 0)
+	if (area == 0) {
 		return 0;
-	else
+	} else {
 		return pos / area;
+	}
 }
 
 double ScrollBar::get_grabber_offset() const {
-
 	return (get_area_size()) * get_as_ratio();
 }
 
 Size2 ScrollBar::get_minimum_size() const {
-
 	Ref<Texture> incr = get_icon("increment");
 	Ref<Texture> decr = get_icon("decrement");
 	Ref<StyleBox> bg = get_stylebox("scroll");
 	Size2 minsize;
 
 	if (orientation == VERTICAL) {
-
 		minsize.width = MAX(incr->get_size().width, (bg->get_minimum_size() + bg->get_center_size()).width);
 		minsize.height += incr->get_size().height;
 		minsize.height += decr->get_size().height;
@@ -515,7 +503,6 @@ Size2 ScrollBar::get_minimum_size() const {
 	}
 
 	if (orientation == HORIZONTAL) {
-
 		minsize.height = MAX(incr->get_size().height, (bg->get_center_size() + bg->get_minimum_size()).height);
 		minsize.width += incr->get_size().width;
 		minsize.width += decr->get_size().width;
@@ -527,21 +514,18 @@ Size2 ScrollBar::get_minimum_size() const {
 }
 
 void ScrollBar::set_custom_step(float p_custom_step) {
-
 	custom_step = p_custom_step;
 }
 
 float ScrollBar::get_custom_step() const {
-
 	return custom_step;
 }
 
 void ScrollBar::_drag_node_exit() {
-
 	if (drag_node) {
 		drag_node->disconnect("gui_input", this, "_drag_node_input");
 	}
-	drag_node = NULL;
+	drag_node = nullptr;
 }
 
 void ScrollBar::_drag_node_input(const Ref<InputEvent> &p_input) {
@@ -552,12 +536,11 @@ void ScrollBar::_drag_node_input(const Ref<InputEvent> &p_input) {
 	Ref<InputEventMouseButton> mb = p_input;
 
 	if (mb.is_valid()) {
-
-		if (mb->get_button_index() != 1)
+		if (mb->get_button_index() != 1) {
 			return;
+		}
 
 		if (mb->is_pressed()) {
-
 			drag_node_speed = Vector2();
 			drag_node_accum = Vector2();
 			last_drag_node_accum = Vector2();
@@ -573,9 +556,7 @@ void ScrollBar::_drag_node_input(const Ref<InputEvent> &p_input) {
 			}
 
 		} else {
-
 			if (drag_node_touching) {
-
 				if (drag_node_speed == Vector2()) {
 					drag_node_touching_deaccel = false;
 					drag_node_touching = false;
@@ -590,19 +571,19 @@ void ScrollBar::_drag_node_input(const Ref<InputEvent> &p_input) {
 	Ref<InputEventMouseMotion> mm = p_input;
 
 	if (mm.is_valid()) {
-
 		if (drag_node_touching && !drag_node_touching_deaccel) {
-
 			Vector2 motion = Vector2(mm->get_relative().x, mm->get_relative().y);
 
 			drag_node_accum -= motion;
 			Vector2 diff = drag_node_from + drag_node_accum;
 
-			if (orientation == HORIZONTAL)
+			if (orientation == HORIZONTAL) {
 				set_value(diff.x);
+			}
 
-			if (orientation == VERTICAL)
+			if (orientation == VERTICAL) {
 				set_value(diff.y);
+			}
 
 			time_since_motion = 0;
 		}
@@ -610,20 +591,17 @@ void ScrollBar::_drag_node_input(const Ref<InputEvent> &p_input) {
 }
 
 void ScrollBar::set_drag_node(const NodePath &p_path) {
-
 	if (is_inside_tree()) {
-
 		if (drag_node) {
 			drag_node->disconnect("gui_input", this, "_drag_node_input");
 			drag_node->disconnect("tree_exiting", this, "_drag_node_exit");
 		}
 	}
 
-	drag_node = NULL;
+	drag_node = nullptr;
 	drag_node_path = p_path;
 
 	if (is_inside_tree()) {
-
 		if (has_node(p_path)) {
 			Node *n = get_node(p_path);
 			drag_node = Object::cast_to<Control>(n);
@@ -637,7 +615,6 @@ void ScrollBar::set_drag_node(const NodePath &p_path) {
 }
 
 NodePath ScrollBar::get_drag_node() const {
-
 	return drag_node_path;
 }
 
@@ -654,7 +631,6 @@ bool ScrollBar::is_smooth_scroll_enabled() const {
 }
 
 void ScrollBar::_bind_methods() {
-
 	ClassDB::bind_method(D_METHOD("_gui_input"), &ScrollBar::_gui_input);
 	ClassDB::bind_method(D_METHOD("set_custom_step", "step"), &ScrollBar::set_custom_step);
 	ClassDB::bind_method(D_METHOD("get_custom_step"), &ScrollBar::get_custom_step);
@@ -667,11 +643,10 @@ void ScrollBar::_bind_methods() {
 }
 
 ScrollBar::ScrollBar(Orientation p_orientation) {
-
 	orientation = p_orientation;
 	highlight = HIGHLIGHT_NONE;
 	custom_step = -1;
-	drag_node = NULL;
+	drag_node = nullptr;
 
 	drag.active = false;
 
@@ -684,8 +659,9 @@ ScrollBar::ScrollBar(Orientation p_orientation) {
 	target_scroll = 0;
 	smooth_scroll_enabled = false;
 
-	if (focus_by_default)
+	if (focus_by_default) {
 		set_focus_mode(FOCUS_ALL);
+	}
 	set_step(0);
 }
 
